<?php
abstract class Item {
	
	private $id;
	static private $lastId = 0;

	protected $data;
	
	public function __construct() {
		$this->id = self::$lastId++;
		$this->data = array();
	}

	/**
	 * magicka funkce na nastaveni hodnoty "neexistujici" clenske promenne
	 * @param $key - nazev promenne
	 * @param $value - hodnota promenne
	 * @return nevraci nic
	 */
	public function __set($key, $value) {
		if (strcmp($key, 'id') === 0)
			return;
		else
			$this->data[$key] = $value;
	}
	
	/**
	 * magicka funkce na ziskani hodnoty "neexistujici" clenske promenne
	 * @param $key - nazev promenne
	 * @return hodnotu promenne nebo null, pokud neexistuje
	 */
	public function __get($key) {
		if (strcmp($key, 'id') === 0)
			return $this->id;
		else if (isset($this->data[$key]))
			return $this->data[$key];
		else
			return null;
	}
	
	/**
	 * kdyz uz se implementuji magicke __set a __get, je vhodne implementovat
	 * i __isset - aby pomoci isset() slo testovat, zda je promenna nastavena
	 * @param $key - nazev promenne
	 * @return true, pokud je nastavena, false jinak
	 */
	public function __isset($key) {
		if (strcmp($key, 'id') === 0)
			return true;
		return isset($this->data[$key]);
	}
	
	/**
	 * kdyz uz se implementuji magicke __set a __get, je vhodne implementovat
	 * i __unset - aby pomoci unset() slo promennou odstranit
	 * @param $key - nazev promenne
	 */
	public function __unset($key) {
		if (strcmp($key, 'id') === 0)
			return ;
		unset($this->data[$key]);
	}
	
	public function __toString() {
		$result = 'Class: '.get_class($this).'<br />'.
		          'Id: '.$this->id.'<br />';
		foreach ($this->data as $item) 
			$result .= $item.'<br />';
		return $result;
	}
	
	function getId() {
		return $this->id;
	}
}
?>